@@ -6,9 +6,39 @@ from parsers import parse_tweet
 from defaults import default_kws
 from connection import connect 
 
+try:
+  import gevent
+  from gevent.pool import Pool
+
+  # patch everything except for thread.
+  import gevent.monkey
+  gevent.monkey.patch_socket()
+  gevent.monkey.patch_ssl()
+  gevent.monkey.patch_os()
+  gevent.monkey.patch_select()
+  gevent.monkey.patch_subprocess()
+
+except ImportError:
+  imported_gevent = False
+else:
+  imported_gevent = True
+
 logging.basicConfig()
 logger = logging.getLogger("birdfeeder")
 
+class TimeoutError(Exception):
+  pass
+
+def concurrent_yield(func, iterartor, **kw):
+  if imported_gevent:
+    p = Pool(kw.get('num_workers'))
+    for result in p.imap_unordered(func, iterartor):
+      yield result
+  else:
+    logger.warn('Cannot run concurrently without importing gevent')
+    for item in iterartor:
+      yield func(item)
+
 def validate_kw(kw, requires):
   """
   Validate kw.
@@ -62,6 +92,7 @@ def catch_err(func, api, **kw):
   timeout = kw.get('timeout')
 
   # try until we timeout
+  t0 = time.time()
   while True:
     try:
       tweets = func(api, **kw)
@@ -69,14 +100,15 @@ def catch_err(func, api, **kw):
     
     # backoff from errors
     except twython.exceptions.TwythonError as e:
-      t0 = time.time()
       time.sleep(wait)
       wait *= backoff
 
       # timeout
       now = time.time()
       if now - t0 > timeout:
-        logger.warn("Timing out beacause of {0}".format(e))
+        err_msg = "Timing out beacause of {0}".format(e)
+        logger.error(err_msg)
+        # raise TimeoutError(err_msg)
         tweets = []
         break
 
